package org.jeneva.validation;

import java.util.Collection;

import org.jeneva.Dtobase;

/**
 * defines validation checking routines
 */
public interface IChecker {

	/**
	 * checks if field value is assigned
	 */
	boolean isAssigned(String fieldName, Dtobase target);

	/**
	 * checks if field value was correctly parsed
	 */
	boolean isValidFormat(String fieldName, Dtobase target);

	/**
	 * checks if field value is NULL
	 */
	boolean isNull(Object value);

	/**
	 * checks if field value equals to m
	 */
	boolean isEqualTo(Object m, Object value);

	/**
	 * checks if field value is equal to one of the values
	 */
	boolean isEqualToOneOf(Object[] values, Object value);

	/**
	 * checks if field value is empty string
	 */
	boolean isEmptyString(String value);

	/**
	 * checks if field value length is between min and max values
	 */
	boolean isLengthBetween(int min, int max, String value);

	/**
	 * checks if field value is less than or equal to 'm'. Comparable<?>.
	 */
	boolean isLessOrEqualTo(Object m, @SuppressWarnings("rawtypes") Comparable value);

	/**
	 * checks if field value is less than 'm'. Comparable<?>.
	 */
	boolean isLessThan(Object m, @SuppressWarnings("rawtypes") Comparable value);

	/**
	 * checks if field value is greater than or equal to 'm'. Comparable<?>.
	 */
	boolean isGreaterOrEqualTo(Object m, @SuppressWarnings("rawtypes") Comparable value);

	/**
	 * checks if field value is greater than 'm'. Comparable<?>.
	 */
	boolean isGreaterThan(Object m, @SuppressWarnings("rawtypes") Comparable value);

	/**
	 * checks if field value collection size is between min and max values inclusively (field must implement Comparable<?>)
	 */
	boolean isCountBetween(int min, int max, Collection<?> value);

	/**
	 * checks if field value qualifies to regular expression
	 */
	boolean isRegexpr(String expr, String value);
}