package org.jeneva.validation.impl;

import java.util.Collection;

import org.jeneva.Dtobase;
import org.jeneva.validation.IChecker;

@SuppressWarnings("unchecked")
public class Checker implements IChecker {

	/**
	 * checks if field value is assigned
	 */
	@Override
	public boolean isAssigned(String fieldName, Dtobase target) {
		return target.isFieldAssigned(fieldName);
	}

	/**
	 * checks if field value was correctly parsed
	 */
	@Override
	public boolean isValidFormat(String fieldName, Dtobase target) {
		return !target.isFieldWrong(fieldName);
	}

	/**
	 * checks if field value is NULL
	 */
	@Override
	public boolean isNull(Object value) {
		return null == value;
	}

	/**
	 * checks if field value equals to m
	 */
	@Override
	public boolean isEqualTo(Object m, Object value) {
		return m.equals(value);
	}

	/**
	 * checks if field value is equal to one of values
	 */
	@Override
	public boolean isEqualToOneOf(Object[] values, Object value) {
		boolean success = false;
		for (int i = 0; i < values.length; i++)
		{
			if (value.equals(values[i]))
			{
				success = true;
				break;
			}
		}

		return success;
	}

	/**
	 * checks if field value is empty string
	 */
	@Override
	public boolean isEmptyString(String value) {
		return value.equals("");
	}

	/**
	 * checks if field value length is between min and max values
	 */
	@Override
	public boolean isLengthBetween(int min, int max, String value) {
		return value.length() >= min && value.length() <= max;
	}

	/**
	 * checks if field value is less than or equal to 'm'
	 */
	@Override
	public boolean isLessOrEqualTo(Object m, @SuppressWarnings("rawtypes") Comparable value) {
		return value.compareTo(m) <= 0;
	}

	/**
	 * checks if field value is less than 'm'. Comparable.
	 */
	@Override
	public boolean isLessThan(Object m, @SuppressWarnings("rawtypes") Comparable value) {
		return value.compareTo(m) < 0;
	}

	/**
	 * checks if field value is greater than or equal to 'm'. Comparable.
	 */
	@Override
	public boolean isGreaterOrEqualTo(Object m, @SuppressWarnings("rawtypes") Comparable value) {
		return value.compareTo(m) >= 0;
	}

	/**
	 * checks if field value is greater than 'm'. Comparable.
	 */
	@Override
	public boolean isGreaterThan(Object m, @SuppressWarnings("rawtypes") Comparable value) {
		return value.compareTo(m) > 0;
	}

	/**
	 * checks if field value collection size is between min and max values inclusively (field must implement Collection)
	 */
	@Override
	public boolean isCountBetween(int min, int max, Collection<?> value) {
		return value.size() >= min && value.size() <= max;
	}

	/**
	 * checks if field value qualifies to regular expression
	 */
	@Override
	public boolean isRegexpr(String expr, String value) {
		return value.matches(expr);
	}
}